" nvim-lspconfig
sign define DiagnosticSignError text= texthl=DiagnosticSignError linehl= numhl=
sign define DiagnosticSignWarn text= texthl=DiagnosticSignWarn linehl= numhl=
sign define DiagnosticSignInfo text= texthl=DiagnosticSignInfo linehl= numhl=
sign define DiagnosticSignHint text= texthl=DiagnosticSignHint linehl= numhl=

set completeopt=menuone,noinsert,noselect

lua << EOF
lsp_mappings = function(bufnr)
  -- Mappings.
  local opts = { noremap=true, silent=true, buffer=bufnr }
  vim.keymap.set("n", "gD", vim.lsp.buf.declaration, opts)
  vim.keymap.set("n", "gd", vim.lsp.buf.definition, opts)
  vim.keymap.set("n", "K", vim.lsp.buf.hover, opts)
  vim.keymap.set("n", "gi", vim.lsp.buf.implementation, opts)
  vim.keymap.set("n", "<c-k>", vim.lsp.buf.signature_help, opts)

  vim.keymap.set("n", "<space>wa", vim.lsp.buf.add_workspace_folder, opts)
  vim.keymap.set("n", "<space>wr", vim.lsp.buf.remove_workspace_folder, opts)
  vim.keymap.set("n", "<space>wl", function() vim.pretty_print(vim.lsp.buf.list_workspace_folders()) end, opts)
  vim.keymap.set("n", "<space>D", vim.lsp.buf.type_definition, opts)
  vim.keymap.set("n", "<space>rn", vim.lsp.buf.rename, opts)

  vim.keymap.set("n", "gr", vim.lsp.buf.references, opts)
  vim.keymap.set("n", "g0", vim.lsp.buf.document_symbol, opts)
  vim.keymap.set("n", "gW", vim.lsp.buf.workspace_symbol, opts)

  vim.keymap.set("n", "<space>e", vim.diagnostic.open_float, opts)
  vim.keymap.set("n", "[d", vim.diagnostic.goto_prev, opts)
  vim.keymap.set("n", "]d", vim.diagnostic.goto_next, opts)
  vim.keymap.set("n", "<space>q", vim.diagnostic.setloclist, opts)
end
EOF

" lspkind-nvim
lua cmp_format = require("lspkind").cmp_format {
\   mode = "symbol",
\   preset = "codicons",
\   menu = {nvim_lsp = "[LSP]", buffer = "[Buffer]", path = "[Path]"}
\ }

lua << EOF
-- This section must be before setting up any LSP server for the automatic
-- installation of servers to work as expected.
require("mason").setup()  -- Just for updating paths to mason packages on launch
require("mason-lspconfig").setup({ automatic_installation = true })
EOF

lua << EOF
local nvim_lsp = require("lspconfig")

local on_attach = function(client, bufnr)
  vim.api.nvim_create_autocmd("CursorHoldI", {
    group = vim.api.nvim_create_augroup("Diagnostics", { clear = true }),
    buffer = bufnr,
    callback = function()
      vim.diagnostic.open_float({ focusable = false, header = "" })
    end
  })

  vim.lsp.handlers["textDocument/publishDiagnostics"] = vim.lsp.with(
    vim.lsp.diagnostic.on_publish_diagnostics, {
      update_in_insert = true,
      virtual_text = { spacing = 24 }
    }
  )
  vim.diagnostic.config({
    float = { border = "rounded" }
  })

  local cmp = require("cmp")
  cmp.setup {
    mapping = {
      ["<C-n>"] = cmp.mapping.select_next_item({ behavior = cmp.SelectBehavior.Insert }),
      ["<C-p>"] = cmp.mapping.select_prev_item({ behavior = cmp.SelectBehavior.Insert }),
      ["<CR>"] = cmp.mapping({
        i = cmp.mapping.confirm({ behavior = cmp.ConfirmBehavior.Replace, select = true }),
        c = function(fallback)
          if cmp.visible() then
            cmp.confirm({ behavior = cmp.ConfirmBehavior.Replace, select = true })
          else
            fallback()
          end
        end
      }),
      ["<C-Space>"] = cmp.mapping(cmp.mapping.complete(), { "i", "c" }),
    },
    formatting = {format = cmp_format},
    sources = {
      {name = "nvim_lsp"},
      {name = "buffer", keyword_length = 5},
      {name = "path"}
    },
    experimental = {ghost_text = true},
  }
  lsp_mappings(bufnr)
end

local capabilities = vim.lsp.protocol.make_client_capabilities()
capabilities = require("cmp_nvim_lsp").default_capabilities(capabilities)

local settings = require("lsp_server_settings")
local servers = {"pylsp", "clangd"}
for _, lsp in ipairs(servers) do
  nvim_lsp[lsp].setup {on_attach=on_attach, settings={[lsp]=settings[lsp]},
                       capabilities=capabilities}
end
EOF

" nvim-cmp
autocmd FileType gitcommit lua require("cmp").setup.buffer {
\   sources = {
\     { name = "spell" },
\     { name = "buffer", keyword_length = 5 },
\     { name = "path" },
\   },
\ }
